#!python
# -*- mode: python; Encoding: utf-8; coding: utf-8 -*-
# Last updated: <2024/02/24 21:28:38 +0900>
"""
font png image to FreeBASIC header file

Usage: python fontpng2bitsbas.py -i input.png --label label_name > image.bi

Windows10 x64 22H2 + Python 3.10.10 64bit + Pillow 10.1.0
"""

import argparse
import os
import sys
from PIL import Image


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("-i", "--infile", required=True, help="PNG image file")
    parser.add_argument("--label", help="symbol name")
    args = parser.parse_args()
    infile = args.infile

    if not os.path.isfile(infile):
        print("Error: Not found %s" % infile)
        sys.exit()

    if not args.label:
        label = infile.replace(".", "_")
        label = label.replace(" ", "_")
        label = label.replace("-", "_")
    else:
        label = args.label

    im = Image.open(infile).convert("L")
    im.point(lambda x: 0 if x < 128 else x)
    imgw, imgh = im.size
    w = int(imgw / 16)
    h = int(imgh / 6)
    
    alen = int(w / 8) + (1 if (w % 8) != 0 else 0)
    clen = alen * h
    cnum = 16 * 6

    obuf = []

    for c in range(16 * 6):
        bx = int(c % 16) * w
        by = int(c / 16) * h

        obuf.append("  { _")
        # obuf.append("    ' code = &H%02x" % (c + 0x20))

        for y in range(h - 1, -1, -1):
            buf = []
            for i in range(alen):
                buf.append(0)

            for x in range(w):
                v = im.getpixel((bx + x, by + y))
                if v > 128:
                    buf[int(x / 8)] |= 1 << (7 - (x % 8))

            s = ""
            for v in buf:
                s += "&H%s, " % format(v, "02x")

            if y <= 0:
                s = s.rstrip(" ,")
                
            obuf.append("    %s _" % s)

        if c >= (16 * 6 - 1):
            obuf.append("  } _")
        else:
            obuf.append("  }, _")

    print("' infile: %s" % infile)
    print("' source image size = %d x %d\n" % (imgw, imgh))

    print("#ifndef __%s__" % label.upper())
    print("#define __%s__\n" % label.upper())
    
    print("Const %s_WIDTH = %d" % (label.upper(), w))
    print("Const %s_HEIGHT = %d" % (label.upper(), h))
    print("Const %s_CHR_LEN = %d\n" % (label.upper(), clen))

    print("Dim Shared %s(%d - 1, %d - 1) As UByte => { _" % (label, cnum, clen))
    
    for s in obuf:
        print(s)

    print("}")
    print("#endif")


if __name__ == "__main__":
    main()
